; The GIMP -- an image manipulation program
; Copyright (C) 1995 Spencer Kimball and Peter Mattis
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;__________________________________________________________________________

; Mark Number Circles -- GIMP script to make sequencial number along a path
; Copyright (C) 2011 Silas Silva. 2021 AlSchemist

; This program tries to solve a long-term problem for users of image processing
; programs.  Sometimes it is necessary to "label" (or "mark") parts of a picture
; to be referenced in a document.  It is hard and repetitive to draw a circle, a
; number within it, etc., so I developed this script.

; Usage: To work with it, it is first necessary to draw a path on the image, with the
; vector/path tool, then call the Mark Number Circles script that will create one mark for node on the path.

; Main function. Require one parameter: the image to work with.
(define	(mark-number-circles image)
	(if (zero? (car (gimp-image-get-vectors image)))
		(error "mark-number-circles needs two or more points created with the Paths Tool!")
		(let*	(	(pathId		(vector-ref (cadr (gimp-image-get-vectors image)) 0))
					(strokeId	(vector-ref (cadr (gimp-vectors-get-strokes pathId)) 0))
					(vPoint		(car (drop (gimp-vectors-stroke-get-points pathId strokeId) 2)))
					(radius 30) ; the radius is a constant
					(layerMain	(car (gimp-image-get-active-layer image)))
				)
			(gimp-context-push)					(gimp-image-undo-group-start image)
			(gimp-context-set-foreground "#ffffff") ; Was "#4C711D"
			(gimp-context-set-background "#000099") ; Paint selection. Was "#ffffff"
			(make-circles vPoint radius image)
			(gimp-image-set-active-layer image layerMain)
			(gimp-image-undo-group-end image)	(gimp-context-pop)
)	)	)
; Make all circles. lstCoord is the list of coordinates.
; It is the vector returned by gimp-path-list, but transformed in a list with vector->list.
; See mark-number-circles function for detais.
; radius is the circle radius and image is the image to work with.
(define	(make-circles vPoint radius image)
	(let*	(	(fontsize	(/ radius 1.75)) (radius/2	(/ radius 2)) (BG-IMAGE-FILL 1) (REPLACE 2)
				(lenv		(- (vector-length vPoint) 1))
			)
		; Draw ONE circle. It receives x and y coordinates for the center of the layer
		; (and therefore, the circle), the radius of the circle, 
		; the nbr that will be printed inside the circle and the image to work with.
		(define	(make-circle x y nbr)
			(let	(	(layer	(car (gimp-layer-new image radius radius RGBA-IMAGE "Circle" 100 NORMAL-MODE))) ; Create a new layer
						(x0 (- x radius/2)) ; Define x0 and y0 the top-left coordinate 
						(y0 (- y radius/2)) ; as to make it be in the center of x y
						(strNbr (number->string nbr))
					)
				(gimp-layer-set-offsets layer x0 y0) ; Adjust layer position
				(gimp-image-insert-layer image layer 0 -1)
				(gimp-ellipse-select image x0 y0 radius radius REPLACE TRUE FALSE 0) ; Make a circular selection
				(gimp-edit-fill layer BG-IMAGE-FILL)
				(gimp-layer-set-name layer strNbr)
				(let*	(	(layerTxt (car (gimp-text-fontname image layer 0 0 strNbr 0 TRUE fontsize PIXELS "Arial"))) ; Was: "Sans Bold". Creates text
							(width  (car (gimp-drawable-width layerTxt))) ; Get text width and height
							(height (car (gimp-drawable-height layerTxt)))
							(xt (- x (/ width  2) (if (< nbr 10) 0 1))) ; Discover xt and yt the toplevel coordinates 
							(yt (- y (/ height 2))) ; to make text be in the center of the circle.
						)
					(gimp-layer-set-offsets layerTxt xt yt) ; Position text.
					(gimp-floating-sel-anchor layerTxt)
		)	)	)
		(let loop ((nbr 1) (idx 0)) ; The nbr variable will hold the number to be printed within the circle
			(if	(< idx lenv)
				(let*	((x (vector-ref vPoint idx)) (y (vector-ref vPoint (+ idx 1)))) ; Extract x and y information
					(make-circle x y nbr)
					(loop (+ nbr 1) (+ idx 6)) ; Was 9. Other items are repetitive for us, so drop them.
	)	)	)	)
	(gimp-displays-flush) ; Flush contents.
) ; Register the script for GIMP:
(script-fu-register "mark-number-circles" "Mark Number Circles" "Produce \"circles\" to be used as numeric marks"
	"Silas Silva <silasdb@gmail.com>" "Copyright (C) 2011 Silas Silva, 2021 AlSchemist" "2021-11-08" ""
	SF-IMAGE    "Image"             0
) ; Register the script in the menu
(script-fu-menu-register "mark-number-circles" "<Image>/Filters/Render/")